/*
								+----------------------------------+
								|                                  |
								|     ***   Bezier cubic   ***     |
								|                                  |
								|   Copyright  -tHE SWINe- 2008   |
								|                                  |
								|         BezierCubic.cpp          |
								|                                  |
								+----------------------------------+
*/

/*
 *	2008-08-21
 *
 *	CBezierCubicSpline is now template parametrized by point type, default is Vector3f
 *
 */

#if 0 // do not compile this; this file shouldn't be used anymore

#include "../CallStack.h"
#include "../Vector.h"
#include <vector>
#include <math.h>
#include "BezierCubic.h"

/*
 *								=== CBezierCubicSpline ===
 */

/*
 *	CBezierCubicSpline::CBezierCubicSpline(const std::vector<Vector3f> &r_point_list)
 *		- default constructor
 *		- r_point_list is list of curve points, it's not copied, only referenced
 *		  and must remain unchanged (arc lengths precalculated in constructor)
 *		- note bezier cubic spline arc is constructed between every fourth point,
 *		  while two points between serve as tangents
 */
CBezierCubicSpline::CBezierCubicSpline(const std::vector<Vector3f> &r_point_list)
	:CSpline<Vector3f>(r_point_list)
{
	_ASSERTE(n_Arc_Num() >= 0);
}

/*
 *	virtual int CBezierCubicSpline::n_Arc_Num() const
 *		- return number of spline arcs
 */
int CBezierCubicSpline::n_Arc_Num() const
{
	//_ASSERTE(!m_point_list.size() || (m_point_list.size() - 1) % 3 == 0);
	return (m_point_list.size())? (m_point_list.size() - 1) / 3 : 0;
}

/*
 *	virtual Vector3f CBezierCubicSpline::v_Arc_Position(float f_t, int n_arc) const
 *		- returns point at position f_t along curve arc n_arc
 */
Vector3f CBezierCubicSpline::v_Arc_Position(float f_t, int n_arc) const
{
	_ASSERTE(n_arc >= 0 && n_arc < n_Arc_Num());

	n_arc *= 3;
	Vector3f v_pt0 = m_point_list[n_arc];
	Vector3f v_pt1 = m_point_list[n_arc + 1];
	Vector3f v_pt2 = m_point_list[n_arc + 2];
	Vector3f v_pt3 = m_point_list[n_arc + 3];
	// get polygon points

	float f_omt = 1 - f_t;
	float f_omt2 = f_omt * f_omt;
	float f_omt3 = f_omt2 * f_omt;
	float f_t2 = f_t * f_t;
	float f_t3 = f_t2 * f_t;
	// calculate 1-t, (1-t)^2, (1-t)^3, t^2, t^3

	float f_weight0 = f_omt3;
	float f_weight1 = 3 * f_t * f_omt2;
	float f_weight2 = 3 * f_t2 * f_omt;
	float f_weight3 = f_t3;
	// calculate Bernstein polynoms

	return v_pt0 * f_weight0 + v_pt1 * f_weight1 +
		v_pt2 * f_weight2 + v_pt3 * f_weight3;
	// interpolate
}

/*
 *	virtual Vector3f CBezierCubicSpline::v_Arc_Derivative(float f_t, int n_arc) const
 *		- returns tangent at position f_t along curve arc n_arc
 */
Vector3f CBezierCubicSpline::v_Arc_Derivative(float f_t, int n_arc) const
{
	_ASSERTE(n_arc >= 0 && n_arc < n_Arc_Num());

	n_arc *= 3;
	Vector3f v_pt0 = m_point_list[n_arc];
	Vector3f v_pt1 = m_point_list[n_arc + 1];
	Vector3f v_pt2 = m_point_list[n_arc + 2];
	Vector3f v_pt3 = m_point_list[n_arc + 3];
	// get polygon points

	float f_omt = 1 - f_t;
	float f_omt2 = f_omt * f_omt;
	float f_omt3 = f_omt2 * f_omt;
	float f_t2 = f_t * f_t;
	float f_t3 = f_t2 * f_t;
	// calculate 1-t, (1-t)^2, (1-t)^3, t^2, t^3

	float f_t_ = 1;
	float f_omt_ = -f_t_;
	float f_omt2_ = f_omt * f_omt_ + f_omt_ * f_omt;
	float f_omt3_ = f_omt2 * f_omt_ + f_omt2_ * f_omt;
	float f_t2_ = f_t * f_t_ + f_t_ * f_t;
	float f_t3_ = f_t2 * f_t_ + f_t2_ * f_t;
	// calculate derivatives of 1-t, (1-t)^2, (1-t)^3, t^2, t^3

	float f_weight0 = f_omt3_;
	float f_weight1 = 3 * (f_t_ * f_omt2 + f_t * f_omt2_);
	float f_weight2 = 3 * (f_t2_ * f_omt + f_t2 * f_omt_);
	float f_weight3 = f_t3_;
	// calculate derivatives of Bernstein polynoms

	return v_pt0 * f_weight0 + v_pt1 * f_weight1 +
		v_pt2 * f_weight2 + v_pt3 * f_weight3;
	// interpolate
}

/*
 *								=== ~CBezierCubicSpline ===
 */

#endif //0
